#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import json
import os
import time

from lib.config_system import get_config
from config import *


def save_new_unl_node(id):
    """
    Saves the new unl.
    """

    nodes_list = get_unl_nodes()

    already_in_list = False

    for element in nodes_list:
        if element == id:
            already_in_list = True

    if not already_in_list:

        nodes_list[id] = {}
        nodes_list[id]["date"] = time.time()

        os.chdir(get_config()["main_folder"])
        with open(UNL_NODES_PATH, "w") as unl_nodes_file:
            json.dump(nodes_list, unl_nodes_file, indent=4)


def get_unl_nodes():
    """
    Returns the UNL nodes list from UNL_NODES_PATH.
    """

    if not os.path.exists(UNL_NODES_PATH):
        return {}

    os.chdir(get_config()["main_folder"])
    with open(UNL_NODES_PATH, "rb") as unl_nodes_file:
        return json.load(unl_nodes_file)


def get_as_node_type(id_list):
    """
    Converts the UNL node list to Node class.
    """

    from node.myownp2pn import mynode

    temp_list = []
    for list_node in id_list:
        for each_node in (
            mynode.main_node.nodes_inbound + mynode.main_node.nodes_outbound
        ):
            if list_node == each_node.id:
                temp_list.append(each_node)

    return temp_list


def node_is_unl(node_id):
    """
    Returns the this node is unl or not.
    """

    for unl in get_unl_nodes():
        temp_unl = unl
        if node_id == temp_unl:
            return True
    return False


def unl_node_delete(node_id):
    """
    Deletes the UNL node
    """

    saved_nodes = get_unl_nodes()
    if node_id in saved_nodes:
        del saved_nodes[node_id]

        os.chdir(get_config()["main_folder"])
        with open(UNL_NODES_PATH, "w") as connected_node_file:
            json.dump(saved_nodes, connected_node_file, indent=4)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import sys

import flask
from flask import jsonify, request
import argparse

from transactions.send_the_coin import send_the_coin
from transactions.save_to_my_transaction import GetMyTransaction

from lib.export import export_the_transactions
from lib.settings_system import the_settings, test_mode, debug_mode
from lib.status import Status

from node.node_connection import ndstart, ndstop, ndconnect, ndconnectmixdb, ndid
from node.unl import save_new_unl_node

from blockchain.block.get_block import GetBlockFromOtherNode
from blockchain.block.create_block import CreateBlock

from wallet.create_a_wallet import create_a_wallet
from wallet.print_wallets import print_wallets
from wallet.wallet_selector import wallet_selector
from wallet.delete_current_wallet import delete_current_wallet
from wallet.print_balance import print_balance


app = flask.Flask(__name__)


@app.route("/wallet/print", methods=["GET"])
def print_wallets_page():
    return jsonify(print_wallets())


@app.route("/wallet/change/<number>", methods=["GET"])
def wallet_change_page(number):
    wallet_selector(number)
    return jsonify(print_wallets())


@app.route("/wallet/create/<password>", methods=["GET"])
def create_wallet_page(password):
    create_a_wallet(password)
    return jsonify(print_wallets())


@app.route("/wallet/delete", methods=["GET"])
def delete_wallets_page():
    delete_current_wallet()
    return jsonify(print_wallets())


@app.route("/send/coin/<address>/<amount>/<password>", methods=["GET"])
def send_coin_page(address, amount, password):
    send_the_coin(address, amount, password)
    return jsonify("OK")


@app.route("/wallet/balance", methods=["GET"])
def balance_wallets_page():
    return jsonify(print_balance())


@app.route("/node/start/<ip>/<port>", methods=["GET"])
def node_start_page(ip, port):
    ndstart(str(ip), int(port))
    return jsonify("OK")


@app.route("/node/stop", methods=["GET"])
def node_stop_page():
    ndstop()
    return jsonify("OK")


@app.route("/node/connect/<ip>/<port>", methods=["GET"])
def node_connect_page(ip, port):
    ndconnect(str(ip), int(port))
    return jsonify("OK")


@app.route("/node/connectmixdb", methods=["GET"])
def node_connectmixdb_page():
    ndconnectmixdb()
    return jsonify("OK")


# /node/newunl/?MFYw......
@app.route("/node/newunl/", methods=["GET"])
def node_newunl_page():
    save_new_unl_node(request.query_string.decode("utf-8"))
    return jsonify("OK")


@app.route("/node/id", methods=["GET"])
def node_id_page():
    return jsonify(ndid())


@app.route("/settings/test/on", methods=["GET"])
def settings_test_on_page():
    test_mode(True)
    return jsonify("OK")


@app.route("/settings/test/off", methods=["GET"])
def settings_test_off_page():
    test_mode(False)
    return jsonify("OK")


@app.route("/settings/debug/on", methods=["GET"])
def settings_debug_on_page():
    app.config["DEBUG"] = True
    debug_mode(True)
    return jsonify("OK")


@app.route("/settings/debug/off", methods=["GET"])
def settings_debug_off_page():
    app.config["DEBUG"] = False
    debug_mode(False)
    return jsonify("OK")


@app.route("/block/get", methods=["GET"])
def block_get_page():
    if the_settings()["test_mode"]:
        CreateBlock()
    else:
        GetBlockFromOtherNode()
    return jsonify("OK")


@app.route("/export/transactions/csv", methods=["GET"])
def export_transaction_csv_page():
    if export_the_transactions():
        return jsonify("OK")
    else:
        return jsonify("You have not a transaction")


@app.route("/export/transactions/json", methods=["GET"])
def export_transaction_json_page():
    return jsonify([i.__dict__ for i in GetMyTransaction()])


@app.route("/status", methods=["GET"])
def status_page():
    return jsonify(Status())


def start():
    parser = argparse.ArgumentParser(
        description="This is an open source decentralized application network. In this network, you can develop and publish decentralized applications."
    )

    parser.add_argument("-p", "--port", type=int, help="Add new UNL node")

    args = parser.parse_args()

    if len(sys.argv) < 2:
        app.run(host="0.0.0.0", port=8000)
    else:
        app.run(host="0.0.0.0", port=args.port)

if __name__ == "__main__":
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import pickle
import time
import os

from lib.mixlib import dprint
from lib.config_system import get_config
from lib.perpetualtimer import perpetualTimer

from node.myownp2pn import mynode
from node.unl import get_unl_nodes, get_as_node_type

from transactions.transaction import Transaction
from transactions.pending_to_validating import PendinttoValidating
from transactions.save_to_my_transaction import SavetoMyTransaction

from accounts.account import Account, save_accounts, save_accounts_part
from accounts.get_balance import GetBalance
from accounts.get_sequance_number import GetSequanceNumber

from blockchain.block.save_block_to_blockchain_db import saveBlockstoBlockchainDB
from blockchain.block.blocks_hash import SaveBlockshash, GetBlockshash, SaveBlockshash_part

from wallet.wallet import (
    Ecdsa,
    PrivateKey,
    PublicKey,
    Wallet_Import,
    Signature
    )

from consensus.consensus_main import consensus_trigger

from app.app_main import apps_starter, app_tigger

from config import TEMP_BLOCK_PATH


class Block:
    def __init__(self, sequance_number, creator):
        # TODO: What to do in case of consensus fails will be added

        self.genesis_time = int(time.time())
        self.start_time = int(time.time())
        self.block_time = 7
        self.block_time_change_time = int(time.time())
        self.block_time_change_block = sequance_number

        self.newly = False 

        self.previous_hash = "0"
        self.sequance_number = sequance_number
        self.empty_block_number = 0

        blocks_hash = [self.previous_hash]
        SaveBlockshash(blocks_hash)
        SaveBlockshash_part([])

        accounts = [
            Account(creator, balance=1000000000)
            ]
        save_accounts(accounts)
        save_accounts_part([])
        self.edited_accounts = []

        self.pendingTransaction = []
        self.validating_list = []
        self.transaction_fee = 0.02
        self.default_transaction_fee = 0.02
        self.default_optimum_transaction_number = 10 # Each user settings by our hardware
        self.default_increase_of_fee = 0.01

        self.hash = None

        self.max_tx_number = 2
        self.minumum_transfer_amount = 1000

        self.raund_1_starting_time = None
        self.raund_1_time = 2.3333333333333335
        self.raund_1 = False
        self.raund_1_node = False

        self.raund_2_starting_time = None
        self.raund_2_time = 2.3333333333333335
        self.raund_2 = False
        self.raund_2_node = False

        self.consensus_timer = 0.50

        self.increase_the_time = 0
        self.increase_the_time_2 = 0
        self.decrease_the_time = 0
        self.decrease_the_time_2 = 0

        self.validated = False
        self.validated_time = None

        self.dowload_true_block = ""

        self.save_block()
        perpetualTimer(self.consensus_timer, consensus_trigger).start()
        apps_starter()

    def reset_the_block(self):
        """
        When the block is verified and if block have a transaction 
        and if block have at least half of the max_tx_number transaction,it saves the block 
        and makes the edits for the new block.
        """

        if self.increase_the_time == 3:
            self.increase_the_time = 0
            self.raund_1_time += 0.1
            self.block_time_change_time = int(time.time())
            self.block_time_change_block = self.sequance_number


        if self.decrease_the_time == 3:
            self.decrease_the_time = 0
            if not self.raund_1_time <= 2:
                self.raund_1_time -= 0.1
                self.block_time_change_time = int(time.time())
                self.block_time_change_block = self.sequance_number


        if self.increase_the_time_2 == 3:
            self.increase_the_time_2 = 0
            self.raund_2_time += 0.1
            self.block_time_change_time = int(time.time())
            self.block_time_change_block = self.sequance_number


        if self.decrease_the_time_2 == 3:
            self.decrease_the_time_2 = 0
            if not self.raund_2_time <= 2:
                self.raund_2_time -= 0.1
                self.block_time_change_time = int(time.time())
                self.block_time_change_block = self.sequance_number


        self.block_time = self.raund_1_time + self.raund_2_time


        #Printing validated block.
        dprint("""\n
  _____                          _     ____  _      ____   _____ _  __
 / ____|                        | |   |  _ \| |    / __ \ / ____| |/ /
| |    _   _ _ __ _ __ ___ _ __ | |_  | |_) | |   | |  | | |    | ' / 
| |   | | | | '__| '__/ _ \ '_ \| __| |  _ <| |   | |  | | |    |  <  
| |___| |_| | |  | | |  __/ | | | |_  | |_) | |___| |__| | |____| . \ 
 \_____\__,_|_|  |_|  \___|_| |_|\__| |____/|______\____/ \_____|_|\_\
                                        
        """+str(self.__dict__)+"\n")

        self.start_time = int(time.time())

        self.raund_1_starting_time = None
        self.raund_1 = False
        self.raund_1_node = False

        self.raund_2_starting_time = None
        self.raund_2 = False
        self.raund_2_node = False

        self.validated = False

        

        # Resetting the node candidate blocks.
        for node in get_as_node_type(get_unl_nodes()):
            node.candidate_block = None
            node.candidate_block_hash = None

        if not len(self.validating_list) == 0 and not len(self.validating_list) < (self.max_tx_number / 2):

            
            app_tigger(self)

            my_address = Wallet_Import(-1, 3)
            for tx in self.validating_list:
                if tx.toUser == my_address:
                    SavetoMyTransaction(tx)

            
            saveBlockstoBlockchainDB(self)

            # Resetting and setting the new elements.
            self.previous_hash = self.hash
            current_blockshash_list = GetBlockshash()
            current_blockshash_list.append(self.previous_hash)
            SaveBlockshash(current_blockshash_list)
            self.sequance_number = self.sequance_number + 1
            self.validating_list = []
            self.hash = None

            #Printing new block.
            dprint("""\n
    _   _                 ____  _      ____   _____ _  __
    | \ | |               |  _ \| |    / __ \ / ____| |/ /
    |  \| | _____      __ | |_) | |   | |  | | |    | ' / 
    | . ` |/ _ \ \ /\ / / |  _ <| |   | |  | | |    |  <  
    | |\  |  __/\ V  V /  | |_) | |___| |__| | |____| . \ 
    |_| \_|\___| \_/\_/   |____/|______\____/ \_____|_|\_\
                                            
            """+str(self.__dict__)+"\n")
        else:
            self.empty_block_number += 1



        # Adding self.pendingTransaction to the new/current block.
        PendinttoValidating(self)

        # Saving the new block.
        self.save_block()

    def propagating_the_tx(self, tx):
        """
        Sends the given transaction to UNL nodes.
        """
        items = {
            "transactionrequest": 1,
            "sequance_number": tx.sequance_number,
            "signature": tx.signature,
            "fromUser": tx.fromUser,
            "to_user": tx.toUser,
            "data": tx.data,
            "amount": tx.amount,
            "transaction_fee": tx.transaction_fee,
            "transaction_time":tx.time
        }
        for each_node in get_as_node_type(get_unl_nodes()):
            mynode.main_node.send_data_to_node(each_node, items)

    def createTrans(self, sequance_number, signature, fromUser, toUser, transaction_fee, data, amount, transaction_time, transaction_sender=None):

      # Printing the info of tx
      dprint("\nCreating the transaction")
      dprint("***")
      dprint(sequance_number)
      dprint(signature)
      dprint(fromUser)
      dprint(toUser)
      dprint(data)
      dprint(amount)
      dprint("***\n")
      # End

      # Some data
      signature_class = Signature.fromBase64(signature)
      temp_signature = signature_class.toBase64()

      already_got = self.tx_already_got(fromUser, sequance_number, temp_signature)
      # End

      # Validation
      dprint("\nValidation")
      if Ecdsa.verify((str(sequance_number)+str(fromUser)+str(toUser)+str(data)+str(amount)+str(transaction_fee)+str(transaction_time)), signature_class, PublicKey.fromPem(fromUser)) and not amount < self.minumum_transfer_amount and not transaction_fee < self.transaction_fee and not already_got and not (int(time.time()) - transaction_time) > 60:
        dprint("Signature is valid")

        dprint("Getsequancenumber: "+str(GetSequanceNumber(fromUser, self)+1))
        if sequance_number == (GetSequanceNumber(fromUser, self)+1):
          dprint("Sequance number is valid")

          balance = GetBalance(fromUser, self)

          if balance >= (float(amount)+float(transaction_fee)) and (balance - (float(amount)+float(transaction_fee))) > 2:
            dprint("Amount is valid")

            # Local saving
            the_tx = Transaction(
                sequance_number= sequance_number,
                signature=temp_signature,
                fromUser= fromUser,
                toUser=toUser,
                data = data,
                amount = amount,
                transaction_fee= transaction_fee,
                time_of_transaction = transaction_time
            )
            self.pendingTransaction.append(the_tx)
            self.change_transaction_fee()
            self.save_block()
            # End

            self.propagating_the_tx(the_tx)

            return the_tx

      dprint(" Validation end")
      # End

    def tx_already_got(self, fromUser, sequance_number, temp_signature):
        for already_tx in (self.pendingTransaction + self.validating_list):
            if already_tx.signature == temp_signature:
                return True
            if already_tx.fromUser == fromUser:
                for already_tx_parent in (self.pendingTransaction + self.validating_list):
                    if not temp_signature == already_tx_parent.signature:
                        if sequance_number == already_tx_parent.sequance_number:
                            return True

        return False

    def send_my_response_on_transaction(self, temp_transaction, response, transaction_sender):
        items = {
            "transactionresponse": 1,
            "fromUser": mynode.main_node.id,
            "response": response,
            "transaction_signature": temp_transaction.signature,
            "signature": Ecdsa.sign(
                response+str(temp_transaction.signature),
                PrivateKey.fromPem(Wallet_Import(0, 1))
                ).toBase64()
        }
        mynode.main_node.send_data_to_node(transaction_sender, items)

    def save_block(self):
        os.chdir(get_config()["main_folder"])
        with open(TEMP_BLOCK_PATH, 'wb') as block_file:
            pickle.dump(self, block_file, protocol=2)

    def change_transaction_fee(self):
        """
        Increase transaction fee by 0.01 DNC for each self.default_optimum_transaction_number argument
        """
        if not (len(self.pendingTransaction + self.validating_list) // self.default_optimum_transaction_number) == 0:
            increase = (len(self.pendingTransaction + self.validating_list) // self.default_optimum_transaction_number) * self.default_increase_of_fee
            self.transaction_fee += increase
        else:
            self.transaction_fee = self.default_transaction_fee



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from node.myownp2pn import mynode, connectionfrommixdb
from wallet.wallet import Wallet_Import

from config import *


def ndstart(ip, port):
    """
    Starts the node server.
    """

    node = mynode(ip, port)
    node.start()
    return node


def ndstop():
    """
    Stops the node server
    """

    mynode.main_node.stop()


def ndconnect(ip, port):
    """
    Connects to a node.
    """

    mynode.main_node.connect_to_node(ip, port)


def ndconnectmixdb():
    """
    Connects to nodes from mixdb.
    """

    connectionfrommixdb()


def ndid():
    """
    Returns the our node id.
    """

    return "".join(
        [
            l.strip()
            for l in Wallet_Import(0, 0).splitlines()
            if l and not l.startswith("-----")
        ]
    )



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from lib.mixlib import dprint
from lib.merkle_root import MerkleTree

from blockchain.block.blocks_hash import GetBlockshash, GetBlockshash_part, SaveBlockshash_part

from accounts.account import GetAccounts, GetAccounts_part, save_accounts


def CalculateHash(block):
    """
    Calculates and returns the hash of the block.
    """

    # Transaction
    tx_list = []
    dprint(len(block.validating_list))
    for element in block.validating_list[:]:
        dprint(element)
        tx_list.append(element.signature)
    if len(tx_list) != 0:
        tx_hash = MerkleTree(tx_list).getRootHash()
    else:
        tx_hash = "0"


    part_amount = 100000

    # Blocks Hash
    blocks_hash_list = []

    part_of_blocks_hash = GetBlockshash_part()
    the_blocks_hash = GetBlockshash()


    if not len(the_blocks_hash) - (len(part_of_blocks_hash) * part_amount) == part_amount:
        for will_added_blocks_hash in the_blocks_hash[(len(part_of_blocks_hash) * part_amount):]:
            blocks_hash_list.append(will_added_blocks_hash)
    else:
        part_of_blocks_hash.append(MerkleTree(the_blocks_hash[(len(part_of_blocks_hash) * part_amount):]).getRootHash())
        SaveBlockshash_part(part_of_blocks_hash)

    
    for part_of_blocks_hash_element in part_of_blocks_hash:
        blocks_hash_list.append(part_of_blocks_hash_element)
    

    blockshash_hash = MerkleTree(blocks_hash_list).getRootHash()


    # Account
    account_list = []

    part_of_account = GetAccounts_part()
    the_accounts = GetAccounts()

    if not len(the_accounts) - (len(part_of_account) * part_amount) >= part_amount:
        for will_added_accounts in the_accounts[(len(part_of_account) * part_amount):]:
            account_list.append(str(will_added_accounts.dump_json()))
    else:
        part_of_account.append(MerkleTree(the_accounts[(len(part_of_account) * part_amount):]).getRootHash())
        save_accounts(part_of_account)

    
    for part_of_account_element in part_of_account:
        account_list.append(part_of_account_element)

    for edited_account in block.edited_accounts:
        account_list.append(str(edited_account.dump_json()))

    block.edited_accounts.clear()
    
    ac_hash = MerkleTree(account_list).getRootHash()


    # Other elements
    main_list = []
    main_list.append(block.previous_hash)
    main_list.append(str(block.sequance_number))
    main_list.append(blockshash_hash)
    main_list.append(ac_hash)
    main_list.append(tx_hash)
    main_list.append(str(block.default_transaction_fee))
    main_list.append(str(block.default_increase_of_fee))
    main_list.append(str(block.default_optimum_transaction_number))

    # Calculating and returning
    return MerkleTree(main_list).getRootHash()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from wallet.wallet import Wallet_Import

from blockchain.block.get_block import GetBlock

from accounts.get_balance import GetBalance


def print_balance():
    """
    Prints the current wallet balance.
    """

    balance = GetBalance(Wallet_Import(-1, 0), GetBlock())
    print(balance)
    return balance



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import unittest


class Test_Node(unittest.TestCase):

    def test_node_connection_saving_and_unl(self):

        password = "123"

        temp_private_key = Wallet_Create(password)


        node_1 = ndstart("127.0.0.1",10001)

        temp_private_key2 = Wallet_Create(password)

        node_2 = ndstart("127.0.0.1",10002)

        save_new_unl_node(node_1.id)
        save_new_unl_node(node_2.id)

        ndconnect("127.0.0.1", 10001)

        finded_node = False
        in_unl_list = False
        get_as_node = False



        nodes_list = get_connected_node()

        for element in nodes_list:
            if element == node_1.id or element == node_2.id:
                finded_node = True

                temp_unl_node_list = get_unl_nodes()
                temp_get_as_node_type = get_as_node_type(temp_unl_node_list)
                for unl_element in temp_unl_node_list:
                    if unl_element == node_1.id or unl_element == node_2.id:
                        for node_element_of_unl in temp_get_as_node_type:
                            if node_1.host == node_element_of_unl.host or node_2 == node_element_of_unl.host:
                                if node_1.port == node_element_of_unl.port or node_2 == node_element_of_unl.port:
                                    get_as_node = True
                        in_unl_list = True
                        unl_node_delete(unl_element)
                connected_node_delete(element)

        node_1.stop()
        node_2.stop()

        saved_wallets = get_saved_wallet()

        for each_wallet in saved_wallets:
            if temp_private_key == saved_wallets[each_wallet]["privatekey"] or temp_private_key2 == saved_wallets[each_wallet]["privatekey"] :
                Wallet_Delete(each_wallet)

        self.assertEqual(finded_node,True,"Problem on connection saving system.")
        self.assertEqual(in_unl_list,True,"Problem on UNL node saving system.")
        self.assertEqual(get_as_node,True,"Problem on UNL get as node system.")



import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "..","src"))
from wallet.wallet import Wallet_Create, get_saved_wallet, Wallet_Import, Wallet_Delete
from node.node_connection import ndstart, ndconnect, ndstop
from node.node import get_connected_node, connected_node_delete
from node.unl import save_new_unl_node, get_unl_nodes, get_as_node_type, unl_node_delete
unittest.main(exit=False)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from hashlib import sha256

from wallet.wallet import Wallet_Import

from transactions.send_coin import send_coin

from blockchain.block.get_block import GetBlock

from lib.settings_system import the_settings


def send_the_coin(receiver, temp_coin_amount, password):
    try:
        temp_coin_amount = float(temp_coin_amount)
    except ValueError:
        print("This is not float coin amount.")
        return False

    if not temp_coin_amount < GetBlock().minumum_transfer_amount:
        if Wallet_Import(int(the_settings()["wallet"]), 2) == sha256(password.encode("utf-8")).hexdigest():
            send_coin(float(temp_coin_amount), receiver, password)
        else:
            print("Password is not correct")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import unittest


class Test_Settings(unittest.TestCase):

    def test_creating_settings(self):
        temp_settings = the_settings()
        self.assertIsNotNone(temp_settings["test_mode"], "A problem on the test_mode.")
        self.assertIsNotNone(temp_settings["debug_mode"], "A problem on the debug_mode.")

    def test_saving_settings(self):
        backup_settings = the_settings()

        temp_settings = the_settings()


        temp_settings["test_mode"] = True
        temp_settings["debug_mode"] = True
        save_settings(temp_settings)



        temp_test_settings = the_settings()
        self.assertEqual(temp_test_settings["test_mode"], True, "A problem on the saving the settings.")
        self.assertEqual(temp_test_settings["debug_mode"], True, "A problem on the saving the settings.")


        temp_test_settings["test_mode"] = False
        temp_test_settings["debug_mode"] = False
        save_settings(temp_test_settings)



        temp_test_settings2 = the_settings()
        self.assertEqual(temp_test_settings2["test_mode"], False, "A problem on the saving the settings.")
        self.assertEqual(temp_test_settings2["debug_mode"], False, "A problem on the saving the settings.")

        temp_test_settings2["test_mode"] = backup_settings["test_mode"]
        temp_test_settings2["debug_mode"] = backup_settings["debug_mode"]
        save_settings(temp_test_settings2)




import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "..","src"))
from lib.settings_system import the_settings, save_settings
unittest.main(exit=False)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from kivy.properties import StringProperty

from kivymd.uix.gridlayout import MDGridLayout
from kivymd.uix.screen import MDScreen


from kivymd.uix.button import MDFlatButton

from kivymd_extensions.sweetalert import SweetAlert
from kivymd.uix.bottomsheet import MDListBottomSheet

from kivy.core.clipboard import Clipboard

from wallet.wallet import Wallet_Create, Wallet_Import, get_saved_wallet, Wallet_Delete

import os

from accounts.get_balance import GetBalance

from blockchain.block.get_block import GetBlock

from lib.settings_system import the_settings, change_wallet

class WalletScreen(MDScreen):
    pass

class Create_Wallet_Box(MDGridLayout):
    cols = 2
class Delete_Wallet_Box(MDGridLayout):
    cols = 2

class WalletBox(MDGridLayout):
    cols = 2
    text = StringProperty()

    wallet_alert_dialog = None
    delete_wallet_alert_dialog = None

    FONT_PATH = f"{os.environ['DECENTRA_ROOT']}/gui_lib/fonts/"

    def reflesh_balance(self):

        self.text = "Balance: "+str(GetBalance(Wallet_Import(-1,0), GetBlock()))

    def show_wallet_alert_dialog(self):
        if not self.wallet_alert_dialog:
            self.wallet_alert_dialog = SweetAlert(
                title="Creating a wallet",
                type="custom",
                auto_dismiss=False,
                content_cls=Create_Wallet_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_wallet_alert_dialog,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                        on_press=self.create_the_wallet
                    )
                ],
            )
        self.wallet_alert_dialog.open()

    def callback_for_menu_items(self, *args):
        if not args[0] == the_settings()["wallet"]:
            change_wallet(int(args[0]))
            self.reflesh_balance()
        Clipboard.copy(Wallet_Import(int(args[0]),3))
        SweetAlert().fire(
            "The address has been copied to your clipboard.",
            type='success',
        )

    def show_example_list_bottom_sheet(self):
        bottom_sheet_menu = MDListBottomSheet(radius=25,radius_from="top")
        data = {}
        all_wallets = list(get_saved_wallet())

        current_wallet = the_settings()["wallet"]
        for wallet in all_wallets:
            number = all_wallets.index(wallet)
            address = Wallet_Import(all_wallets.index(wallet),3)
            if not current_wallet == number:
                data[number] = address
            else:
                data[number] = address + " - CURRENTLY USED"

        for item in data.items():
            bottom_sheet_menu.add_item(
                str(item[0]) +" : "+ item[1],
                lambda x, y=item[0]: self.callback_for_menu_items(y),

            )
        bottom_sheet_menu.open()

    def dismiss_wallet_alert_dialog(self, widget):
        self.wallet_alert_dialog.dismiss()

    def create_the_wallet(self, widget):
        text_list = []
        for obj in self.wallet_alert_dialog.content_cls.children:
            for sub_obj in obj.children:
                Wallet_Create(sub_obj.text)
                self.dismiss_wallet_alert_dialog(widget)

                sub_obj.text = ""

    def Wallet_Create(self):
        self.show_wallet_alert_dialog()


    def dismiss_delete_wallet_alert_dialog(self, widget):
        self.delete_wallet_alert_dialog.dismiss()
    def show_delete_wallet_alert_dialog(self):
        if not self.delete_wallet_alert_dialog:
            self.delete_wallet_alert_dialog = SweetAlert(
                title="Deleting a wallet",
                type="custom",
                auto_dismiss=False,
                content_cls=Delete_Wallet_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_delete_wallet_alert_dialog,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                        on_press=self.delete_the_wallet
                    )
                ],
            )
        self.delete_wallet_alert_dialog.open()

    def Wallet_Delete(self):
        if not the_settings()["wallet"] == 0:
            self.show_delete_wallet_alert_dialog()
        else:
            SweetAlert().fire(
                "First wallet cannot be deleted.",
                type='failure',
            )     

    def delete_the_wallet(self, widget):
        saved_wallets = get_saved_wallet()
        selected_wallet_pubkey = Wallet_Import(int(the_settings()["wallet"]),0)
        for each_wallet in saved_wallets:
            if selected_wallet_pubkey == saved_wallets[each_wallet]["publickey"]:
                change_wallet(0)
                Wallet_Delete(each_wallet)
                self.reflesh_balance()
                self.dismiss_delete_wallet_alert_dialog(widget)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from threading import Timer


class perpetualTimer:
    """
    It trig the functions at time intervals, independent of the main process.

    perpetualTimer class consists of 3 elements:
      * t: The time interval of perpetualTimer.
      * hFunction: The function to be triggered.
      * thread: The threading timer function.
    """

    def __init__(self, t, hFunction):
        self.t = t
        self.hFunction = hFunction
        self.thread = Timer(self.t, self.handle_function)

    def handle_function(self):
        """
        Runs the function.
        """

        self.hFunction()
        self.thread = Timer(self.t, self.handle_function)
        self.thread.start()

    def start(self):
        """
        Cancels the perpetualTimer.
        """

        self.thread.start()

    def cancel(self):
        """
        Cancels the perpetualTimer.
        """

        self.thread.cancel()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from wallet.wallet import get_saved_wallet

from wallet.print_wallets import print_wallets


from lib.settings_system import change_wallet


def wallet_selector(new_wallet_number=None):
    """
    Changes the current wallet.
    """

    all_wallets = list(get_saved_wallet())
    if not len(all_wallets) == 0:

        while True:
            try:
                if new_wallet_number is None:
                    new_wallet = input("Please select wallet: ")
                else:
                    new_wallet = new_wallet_number
                if int(new_wallet) in list(range(len(all_wallets))):
                    change_wallet(int(new_wallet))
                    print("New Wallets:")
                    print_wallets()
                    break
                else:
                    print("There is no such wallet")
                    if not new_wallet_number is None:
                        break
            except:
                print("This is not a number")
    else:
        print("There is no wallet")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from lib.mixlib import dprint


def PendinttoValidating(block):
    """
    Adds transactions to the verification list
    if there are suitable conditions.
    """

    dprint("Pending transactions number: " + str(len(block.pendingTransaction)))
    dprint("Validating transactions number: " + str(len(block.validating_list)))

    if (
        len(block.validating_list) < block.max_tx_number
        and block.raund_1_starting_time is None
    ):
        for tx in block.pendingTransaction[:]:
            if len(block.validating_list) < block.max_tx_number:
                block.validating_list.append(tx)
                block.pendingTransaction.remove(tx)

    dprint(
        "End mining pending transactions number: " + str(len(block.pendingTransaction))
    )
    dprint(
        "End mining validating transactions number: " + str(len(block.validating_list))
    )



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import unittest


class Test_Config(unittest.TestCase):


    def test_getting_and_saving_config(self):
        finded_true_folder = False

        temp_config = get_config()

        os.chdir(temp_config["main_folder"])

        obj = os.scandir()
        for entry in obj :
            if entry.name == "db":
                finded_true_folder = True

        self.assertEqual(finded_true_folder,True,"A problem on the config.")


import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "..","src"))
from lib.config_system import get_config
unittest.main(exit=False)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from wallet.wallet import Address

from blockchain.block.get_block import GetBlock

from accounts.account import GetAccounts


def GetBalance(user, block):
    """
    Returns the users balance.
    """

    balance = -GetBlock().minumum_transfer_amount
    user = "".join(
        [l.strip() for l in user.splitlines() if l and not l.startswith("-----")]
    )
    user = Address(user)
    for Accounts in GetAccounts():

        if Accounts.Address == user:
            balance += Accounts.balance
            break

    for trans in block.pendingTransaction + block.validating_list:
        if user == trans.fromUser:
            balance -= trans.amount + trans.transaction_fee
    return balance



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import csv
import os

from lib.config_system import get_config

from config import MY_TRANSACTION_EXPORT_PATH

from transactions.save_to_my_transaction import GetMyTransaction


def export_to_csv(obj, filename):
    """
    Export a list of objects to a CSV file.
    """

    if not len(obj) == 0:
        os.chdir(get_config()["main_folder"])
        with open(filename, 'w', newline='', encoding='utf-8') as csvfile:
            fieldnames = [i for i in obj[0].__dict__.keys()]
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()
            for obj in obj:
                writer.writerow(obj.__dict__)
        return True
    else:
        return False


def export_the_transactions():
    """
    Export the transactions to a CSV file.
    """

    return export_to_csv(GetMyTransaction(), MY_TRANSACTION_EXPORT_PATH)   



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import pickle



from config import BLOCKS_PATH

from accounts.account import GetAccounts, GetAccounts_part

from blockchain.block.blocks_hash import GetBlockshash, GetBlockshash_part

from wallet.wallet import Wallet_Import


def saveBlockstoBlockchainDB(block):
    """
    Adds the block to the blockchain database
    at BLOCKS_PATH.
    """

    our_tx = False
    my_public_key = "".join(
        [
            l.strip()
            for l in Wallet_Import(-1, 0).splitlines()
            if l and not l.startswith("-----")
        ]
    )
    my_address = Wallet_Import(-1, 3)
    for validated_transaction in block.validating_list:
        if validated_transaction.fromUser == my_public_key or validated_transaction.toUser == my_address:
            our_tx = True

    # If the block is our transaction, then add it to the blockchain database.
    if our_tx:
        #
        with open(BLOCKS_PATH + str(block.sequance_number) + ".block", "wb") as block_file:
            pickle.dump(block, block_file, protocol=2)

        with open(
            BLOCKS_PATH + str(block.sequance_number) + ".accounts", "wb"
        ) as block_file:
            pickle.dump(GetAccounts(), block_file, protocol=2)

        with open(
            BLOCKS_PATH + str(block.sequance_number) + ".accountspart", "wb"
        ) as block_file:
            pickle.dump(GetAccounts_part(), block_file, protocol=2)

        with open(
            BLOCKS_PATH + str(block.sequance_number) + ".blockshash", "wb"
        ) as block_file:
            pickle.dump(GetBlockshash(), block_file, protocol=2)

        with open(
            BLOCKS_PATH + str(block.sequance_number) + ".blockshashpart", "wb"
        ) as block_file:
            pickle.dump(GetBlockshash_part(), block_file, protocol=2)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import pickle
import os

from lib.config_system import get_config

from config import TEMP_BLOCK_PATH


def GetBlock():
    os.chdir(get_config()["main_folder"])
    with open(TEMP_BLOCK_PATH, 'rb') as block_file:
        return pickle.load(block_file)


def GetBlockFromOtherNode():
    from node.myownp2pn import mynode
    from node.unl import get_unl_nodes, get_as_node_type
    node = mynode.main_node
    unl_list = get_as_node_type(get_unl_nodes())
    node.send_data_to_node(unl_list[0], "sendmefullblock")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import time
import sys
import argparse
from getpass import getpass

from config import MY_TRANSACTION_EXPORT_PATH

from transactions.send_the_coin import send_the_coin
from transactions.print_transactions import PrintTransactions

from node.node_connection import ndstart, ndstop, ndconnect, ndconnectmixdb, ndid
from node.unl import save_new_unl_node

from blockchain.block.get_block import GetBlockFromOtherNode
from blockchain.block.create_block import CreateBlock

from lib.mixlib import banner_maker, menu_space, menu_maker, quit_menu_maker, question_maker
from lib.settings_system import the_settings, test_mode, debug_mode
from lib.export import export_the_transactions
from lib.status import Status

from wallet.create_a_wallet import create_a_wallet
from wallet.print_wallets import print_wallets
from wallet.wallet_selector import wallet_selector
from wallet.delete_current_wallet import delete_current_wallet
from wallet.print_balance import print_balance



def show_menu():
    """
    Prints some information and the menu.
    """

    print(banner_maker(sc_name="Decentra Network", description="This is an open source decentralized application network. In this network, you can develop and publish decentralized applications.", author="Decentra Network Developers"))


    print(menu_space() + \
       menu_maker(menu_number="pw", menu_text="Print Wallets")+ \
       menu_maker(menu_number="w", menu_text="Change Wallet")+ \
	   menu_maker(menu_number="cw", menu_text="Create Wallet")+ \
       menu_maker(menu_number="dw", menu_text="Delete Wallet")+ \
	   menu_space() + \
	   menu_maker(menu_number="sc", menu_text="Send Coin")+ \
       menu_space() + \
       menu_maker(menu_number="gb", menu_text="Get Balance")+ \
       menu_space() + \
       menu_maker(menu_number="ndstart", menu_text="Node Start")+ \
       menu_maker(menu_number="ndstop", menu_text="Node Stop")+ \
       menu_maker(menu_number="ndconnect", menu_text="Node Connect")+ \
       menu_maker(menu_number="ndconnectmixdb", menu_text="Node Connect from mixdb")+ \
       menu_maker(menu_number="ndnewunl", menu_text="Add new UNL node")+ \
       menu_maker(menu_number="ndid", menu_text="Print my id")+ \
       menu_space() + \
       menu_maker(menu_number="testmodeon", menu_text="Test mode ON")+ \
       menu_maker(menu_number="testmodeoff", menu_text="Test mode OF")+ \
       menu_maker(menu_number="debugmodeon", menu_text="Debug mode ON")+ \
       menu_maker(menu_number="debugmodeoff", menu_text="Debug mode OF")+ \
       menu_space() + \
       menu_maker(menu_number="exptrcsv", menu_text="Export Transaction as CSV")+ \
       menu_maker(menu_number="returntrs", menu_text="Export Transaction as CSV")+ \
       menu_space() + \
       menu_maker(menu_number="status", menu_text="Prints the status")+ \
       menu_space() + \
       menu_maker(menu_number="getblock", menu_text="Get block From Other Nodes")+ \
       menu_space())


    print(quit_menu_maker(mode="main"))


def menu():
    """
    The main structure of the cli mode, this function prints the menu, 
    listens to the entries, makes the directions.
    """

    animation = ["[■□□□□□□□□□]","[■■□□□□□□□□]", "[■■■□□□□□□□]", "[■■■■□□□□□□]", "[■■■■■□□□□□]", "[■■■■■■□□□□]", "[■■■■■■■□□□]", "[■■■■■■■■□□]", "[■■■■■■■■■□]", "[■■■■■■■■■■]"]

    for i in range(len(animation)):
        time.sleep(0.1)
        sys.stdout.write("\r" + animation[i])
        sys.stdout.flush()    

    while True:
        show_menu()
        choices_input = question_maker(mode="main")

        if choices_input == "pw":
            print_wallets()

        if choices_input == "w":
            wallet_selector()

        if choices_input == "cw":
            create_a_wallet()

        if choices_input == "dw":
            if "y" == input("Are you sure ? (y or n): "):
                delete_current_wallet()
        if choices_input == "sc":
            send_the_coin(input("Please write receiver adress: "), input("Coin Amount (ex. 1.0): "), getpass("Password: "))

        if choices_input == "gb":
            print_balance()
        if choices_input == "help":
            show_menu()
        if choices_input == "ndstart":
            ndstart(str(input("ip: ")), int(input("port: ")))
        if choices_input == "ndstop":
            ndstop()
        if choices_input == "ndconnect":
            ndconnect(str(input("node ip: ")), int(input("node port: ")))

        if choices_input == "ndconnectmixdb":
            ndconnectmixdb()
        if choices_input == "ndnewunl":
            save_new_unl_node(input("Please write ID of the node: "))
        if choices_input == "ndid":
            print(ndid())
        if choices_input == "testmodeon":
            test_mode(True)
        if choices_input == "testmodeoff":
            test_mode(False)
        if choices_input == "debugmodeon":
            debug_mode(True)
        if choices_input == "debugmodeoff":
            debug_mode(False)

        if choices_input == "exptrcsv":
            if export_the_transactions():
                print(f"CSV file created in {MY_TRANSACTION_EXPORT_PATH} directory")
            else:
                print("You have not a transaction")

        if choices_input == "returntrs":
            PrintTransactions()

        if choices_input == "getblock":
            if the_settings()["test_mode"]:
                CreateBlock()
            else:
                GetBlockFromOtherNode()

        if choices_input == "status":
            print(Status())

        if choices_input == "0":
            exit()


def arguments():
    parser = argparse.ArgumentParser(description="This is an open source decentralized application network. In this network, you can develop and publish decentralized applications. Use the menu (-m) or GUI to gain full control and use the node, operation, etc.")

    parser.add_argument('-pw', '--printwallet', action='store_true',
                        help='Print Wallets')

    parser.add_argument('-w', '--wallet', type=int,
                        help='Change Wallet')

    parser.add_argument('-cw', '--createwallet',
                        help='Create wallet')

    parser.add_argument('-dw', '--deletewallet', action='store_true',
                        help='Delete wallet')

    parser.add_argument('-gb', '--getbalance', action='store_true',
                        help='Get Balance')

    parser.add_argument('-ndnunl', '--ndnewunl', type=str,
                        help='Add new UNL node')

    parser.add_argument('-ndid', '--ndid', action='store_true',
                        help='Print my id')

    parser.add_argument('-tmon', '--testmodeon', action='store_true',
                        help='Test Mode On')
    parser.add_argument('-tmoff', '--testmodeoff', action='store_true',
                        help='Test Mode Off')

    parser.add_argument('-dmon', '--debugmodeon', action='store_true',
                        help='Debug Mode On')
    parser.add_argument('-dmoff', '--debugmodeoff', action='store_true',
                        help='Debug Mode Off')

    parser.add_argument('-exptrcsv', '--exporttransactioncsv', action='store_true',
                        help='Exports the transaction as csv')                

    parser.add_argument('-returntrans', '--returntransactions', action='store_true',
                        help='Exports the transaction as csv')   

    parser.add_argument('-st', '--status', action='store_true',
                        help='Exports the transaction as csv')

    parser.add_argument('-m', '--menu', action='store_true',
                        help='An optional boolean for open the menu.') 
    
    args = parser.parse_args()


    if len(sys.argv) < 2:
        parser.print_help()

    if args.printwallet:
        print_wallets()

    if not args.wallet is None:
        wallet_selector(args.wallet)

    if not args.createwallet is None:
        create_a_wallet(args.createwallet)

    if args.deletewallet:
        delete_current_wallet()

    if args.getbalance:
        print_balance()

    if not args.ndnewunl is None:
        save_new_unl_node(args.ndnewunl)

    if args.ndid:
        print(ndid())

    if args.testmodeon:
        test_mode(True)
    if args.testmodeoff:
        test_mode(False)
    if args.debugmodeon:
        debug_mode(True)
    if args.debugmodeoff:
        debug_mode(False)

    if args.exporttransactioncsv:
        if export_the_transactions():
            print(f"CSV file created in {MY_TRANSACTION_EXPORT_PATH} directory")
        else:
            print("You have not a transaction")

    if args.returntransactions:
        PrintTransactions()

    if args.status:
        print(Status())

    if args.menu:
        menu()


def start():
    arguments()

if __name__ == '__main__':
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from wallet.wallet import Wallet_Import, get_saved_wallet

from lib.mixlib import menu_maker
from lib.settings_system import the_settings


def print_wallets():
    """
    Prints the wallets.
    """

    all_wallets = list(get_saved_wallet())
    current_wallet = the_settings()["wallet"]
    print("\nWallets:")
    result = []
    for wallet in all_wallets:
        number = all_wallets.index(wallet)
        address = Wallet_Import(all_wallets.index(wallet), 3)
        if not current_wallet == number:
            text = menu_maker(menu_number=number, menu_text=address)
            print(text)
            result.append(text)
        else:
            text = menu_maker(menu_number=number, menu_text=address + " - CURRENTLY USED")
            print(text)
            result.append(text)
    return result



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import socket
import sys
import time
import threading
import os
import json

from config import CONNECTED_NODE_PATH

from lib.mixlib import dprint

from node.unl import node_is_unl


def get_connected_node():


        sys.path.append(os.path.join(os.path.dirname(__file__), ".."))

        from lib.config_system import get_config

        if not os.path.exists(CONNECTED_NODE_PATH):
            temp_json = {}
            return temp_json


        os.chdir(get_config()["main_folder"])
        with open(CONNECTED_NODE_PATH, 'rb') as connected_node_file:
            return json.load(connected_node_file)


def save_connected_node(host,port,id):

        node_list = get_connected_node()

        already_in_list = False

        for element in node_list:
            if node_list[element]["host"] == host and node_list[element]["port"] == port:
                already_in_list = True

        if not already_in_list:
         node_list[id] = {}
         node_list[id]["host"] = host
         node_list[id]["port"] = port




         

         sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
         
         from lib.config_system import get_config



         os.chdir(get_config()["main_folder"])
         with open(CONNECTED_NODE_PATH, 'w') as connected_node_file:
             json.dump(node_list, connected_node_file, indent=4)




def connectionfrommixdb():
        node_list = get_connected_node()
        from node.myownp2pn import mynode
        for element in node_list:
            mynode.main_node.connect_to_node(node_list[element]["host"], node_list[element]["port"])


def connected_node_delete(node):
    saved_nodes = get_connected_node()
    if node in saved_nodes:
        del saved_nodes[node]
        from lib.config_system import get_config
    

        os.chdir(get_config()["main_folder"])
        with open(CONNECTED_NODE_PATH, 'w') as connected_node_file:
            json.dump(saved_nodes, connected_node_file, indent=4)


class Node_Connection(threading.Thread):


    def __init__(self, main_node, sock, id, host, port):


        super(Node_Connection, self).__init__()

        self.host = host
        self.port = port
        self.main_node = main_node
        self.sock = sock
        self.terminate_flag = threading.Event()


        self.id = id

        self.candidate_block = None
        self.candidate_block_hash = None


        self.EOT_CHAR = 0x04.to_bytes(1, 'big')


        self.info = {}

        save_connected_node(host,port,id)

    def send(self, data, encoding_type='utf-8'):

        if isinstance(data, str):
            self.sock.sendall( data.encode(encoding_type) + self.EOT_CHAR )

        elif isinstance(data, dict):
            try:
                json_data = json.dumps(data)
                json_data = json_data.encode(encoding_type) + self.EOT_CHAR
                self.sock.sendall(json_data)

            except TypeError as type_error:
                dprint('Node System: This dict is invalid')
                dprint(type_error)

            except Exception as e:
                print('Node System: Unexpected Error in send message')
                print(e)

        elif isinstance(data, bytes):
            bin_data = data + self.EOT_CHAR
            self.sock.sendall(bin_data)

        else:
            dprint('Node System: Node System: Datatype used is not valid please use str, dict (will be send as json) or bytes')


    def stop(self):
        self.terminate_flag.set()

    def parse_packet(self, packet):
        try:
            packet_decoded = packet.decode('utf-8')

            try:
                return json.loads(packet_decoded)

            except json.decoder.JSONDecodeError:
                return packet_decoded

        except UnicodeDecodeError:
            return packet

    def run(self):
        self.sock.settimeout(10.0)          
        buffer = b''

        while not self.terminate_flag.is_set():
            chunk = b''

            try:
                chunk = self.sock.recv(4096) 

            except socket.timeout:
                dprint("Node System: Node_Connection: timeout")

            except Exception as e:
                self.terminate_flag.set()
                dprint('Node System: Unexpected error')
                dprint(e)

            # BUG: possible buffer overflow when no EOT_CHAR is found => Fix by max buffer count or so?
            if chunk != b'':
                buffer += chunk
                eot_pos = buffer.find(self.EOT_CHAR)

                while eot_pos > 0:
                    packet = buffer[:eot_pos]
                    buffer = buffer[eot_pos + 1:]

                    self.main_node.message_count_recv += 1
                    self.main_node.message_from_node( self, self.parse_packet(packet) )

                    eot_pos = buffer.find(self.EOT_CHAR)

            time.sleep(0.01)

        # IDEA: Invoke (event) a method in main_node so the user is able to send a bye message to the node before it is closed?

        self.sock.settimeout(None)
        self.sock.close()
        dprint("Node System: Node_Connection: Stopped")

    def set_info(self, key, value):
        self.info[key] = value

    def get_info(self, key):
        return self.info[key]

    def __str__(self):
        return 'Node_Connection: {}:{} <-> {}:{} ({})'.format(self.main_node.host, self.main_node.port, self.host, self.port, self.id)

    def __repr__(self):
        return '<Node_Connection: Node {}:{} <-> Connection {}:{}>'.format(self.main_node.host, self.main_node.port, self.host, self.port)



class Node(threading.Thread):
    


    def __init__(self, host, port, callback=None):
        

        super(Node, self).__init__()


        self.terminate_flag = threading.Event()

        self.host = host
        self.port = port


        self.callback = callback


        self.nodes_inbound = []

        self.nodes_outbound = []


        from wallet.wallet import Wallet_Import
        self.id = "".join([
            l.strip() for l in Wallet_Import(0,0).splitlines()
            if l and not l.startswith("-----")
        ])


        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.init_server()


        self.message_count_send = 0
        self.message_count_recv = 0
        self.message_count_rerr = 0


    def init_server(self):
        print("Node System: Initialisation of the Node on port: " + str(self.port) + " on node (" + self.id + ")")
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind((self.host, self.port))
        self.sock.settimeout(10.0)
        self.sock.listen(1)


    def delete_closed_connections(self):

        for n in self.nodes_inbound:
            if n.terminate_flag.is_set():
                self.inbound_node_disconnected(n)
                n.join()
                del self.nodes_inbound[self.nodes_inbound.index(n)]

        for n in self.nodes_outbound:
            if n.terminate_flag.is_set():
                self.outbound_node_disconnected(n)
                n.join()
                del self.nodes_outbound[self.nodes_inbound.index(n)]

    def send_data_to_nodes(self, data, exclude=[]):

        self.message_count_send = self.message_count_send + 1
        for n in self.nodes_inbound:
            if n in exclude:
                dprint("Node System: Node send_data_to_nodes: Excluding node in sending the message")
            else:
                try:
                    self.send_data_to_node(n, data)
                except: # lgtm [py/catch-base-exception]
                    pass

        for n in self.nodes_outbound:
            if n in exclude:
                dprint("Node System: Node send_data_to_nodes: Excluding node in sending the message")
            else:
                try:
                    self.send_data_to_node(n, data)
                except: # lgtm [py/catch-base-exception]
                    pass

    def send_data_to_node(self, n, data):

        self.message_count_send = self.message_count_send + 1
        self.delete_closed_connections()
        if n in self.nodes_inbound or n in self.nodes_outbound:
            try:
                n.send(data)

            except Exception as e:
                dprint("Node System: Node send_data_to_node: Error while sending data to the node (" + str(e) + ")")
        else:
            dprint("Node System: Node send_data_to_node: Could not send the data, node is not found!")

    def connect_to_node(self, host, port):

        if host == self.host and port == self.port:
            print("Node System: connect_to_node: Cannot connect with yourself!!")
            return False

        # Check if node is already connected with this node!
        for node in self.nodes_outbound:
            if node.host == host and node.port == port:
                print("Node System: connect_to_node: Already connected with this node.")
                return True

        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            dprint("Node System: connecting to %s port %s" % (host, port))
            sock.connect((host, port))

            # Basic information exchange (not secure) of the id's of the nodes!
            sock.send(self.id.encode('utf-8')) # Send my id to the connected node!
            connected_node_id = sock.recv(4096).decode('utf-8') # When a node is connected, it sends it id!

            if node_is_unl(connected_node_id):
                thread_client = self.create_the_new_connection(sock, connected_node_id, host, port)
                thread_client.start()

                self.nodes_outbound.append(thread_client)
                self.outbound_node_connected(thread_client)
            else:
                dprint("Node System: Could not connect with node because node is not unl node.")
        except Exception as e:
            dprint("Node System: TcpServer.connect_to_node: Could not connect with node. (" + str(e) + ")")
        




    def disconnect_to_node(self, node):

        if node in self.nodes_outbound:
            self.node_disconnect_to_outbound_node(node)
            node.stop()
            node.join()
            del self.nodes_outbound[self.nodes_outbound.index(node)]

        else:
            print("Node System: Node disconnect_to_node: cannot disconnect with a node with which we are not connected.")

    def stop(self):

        self.node_request_to_stop()
        self.terminate_flag.set()


    def create_the_new_connection(self, connection, id, host, port):

        return Node_Connection(self, connection, id, host, port)

    def run(self):

        while not self.terminate_flag.is_set():
            try:
                connection, client_address = self.sock.accept()
                
                connected_node_id = connection.recv(4096).decode('utf-8')
                connection.send(self.id.encode('utf-8'))
                if node_is_unl(connected_node_id):
                    thread_client = self.create_the_new_connection(connection, connected_node_id, client_address[0], client_address[1])
                    thread_client.start()

                    self.nodes_inbound.append(thread_client)

                    self.inbound_node_connected(thread_client)
                else:
                    dprint("Node System: Could not connect with node because node is not unl node.")
                
            except socket.timeout:
                pass

            except Exception as e:
                raise e

            time.sleep(0.01)

        print("Node System: Node stopping...")
        for t in self.nodes_inbound:
            t.stop()

        for t in self.nodes_outbound:
            t.stop()

        time.sleep(1)

        for t in self.nodes_inbound:
            t.join()

        for t in self.nodes_outbound:
            t.join()

        self.sock.settimeout(None)   
        self.sock.close()
        print("Node System: Node stopped")

    def outbound_node_connected(self, node):
        dprint("Node System: outbound_node_connected: " + node.id)
 

    def inbound_node_connected(self, node):
        dprint("Node System: inbound_node_connected: " + node.id)
 

    def inbound_node_disconnected(self, node):
        dprint("Node System: inbound_node_disconnected: " + node.id)


    def outbound_node_disconnected(self, node):
        dprint("Node System: outbound_node_disconnected: " + node.id)


    def message_from_node(self, node, data):
        dprint("Node System: message_from_node: " + node.id + ": " + str(data))
        if self.callback is not None:
            self.callback("message_from_node", self, node, data)

    def node_disconnect_to_outbound_node(self, node):
        dprint("Node System: node wants to disconnect with oher outbound node: " + node.id)


    def node_request_to_stop(self):
        dprint("Node System: node is requested to stop!")


    def __str__(self):
        return 'Node: {}:{}'.format(self.host, self.port)

    def __repr__(self):
        return '<Node {}:{} id: {}>'.format(self.host, self.port, self.id)


    



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import time

from lib.mixlib import dprint

from node.unl import get_as_node_type, get_unl_nodes
from node.myownp2pn import mynode

from blockchain.candidate_block.get_candidate_blocks import GetCandidateBlocks


def consensus_round_2(block):
    """
    At this stage of the consensus process,
    The blocks (block hashes) of our and the
    unl nodes are compared and the block accepted
    by 80 percent is approved.

    Inputs:
      * block: The block (class) we want consensus
      round 2 to be done
    """

    unl_nodes = get_unl_nodes()
    if not block.raund_2_node:
        dprint("Raund 2: in get candidate block hashes\n")

        mynode.main_node.send_my_block_hash(get_as_node_type(unl_nodes))
        block.raund_2_node = True
        block.save_block()

    candidate_class = GetCandidateBlocks()
    dprint("Raund 2 Conditions")
    dprint(len(candidate_class.candidate_block_hashes) > ((len(unl_nodes) * 80) / 100))
    dprint((int(time.time()) - block.raund_2_starting_time) < block.raund_2_time)

    if len(candidate_class.candidate_block_hashes) > ((len(unl_nodes) * 80) / 100):

        if not (int(time.time()) - block.raund_2_starting_time) < block.raund_2_time:
            dprint("Raund 2: first ok")
            for candidate_block in candidate_class.candidate_block_hashes[:]:
                tx_valid = 0

                if block.hash == candidate_block["hash"]:
                    tx_valid += 1

                for other_block in candidate_class.candidate_block_hashes[:]:

                    if candidate_block != other_block:
                        if candidate_block["hash"] == other_block["hash"]:
                            tx_valid += 1

                if tx_valid > ((len(unl_nodes) * 80) / 100):

                    dprint("Raund 2: second ok")
                    if block.hash == candidate_block["hash"]:
                        block.validated = True
                        block.validated_time = int(time.time())
                        block.raund_2 = True

                    else:
                        print("Raund 2: my block is not valid")
                        node = mynode.main_node
                        unl_list = get_as_node_type([candidate_block["sender"]])
                        node.send_data_to_node(unl_list[0], "sendmefullblock")
                        block.dowload_true_block = candidate_block["sender"]
                    block.save_block()

        else:
            if not block.decrease_the_time_2 == 3:
                block.decrease_the_time_2 += 1
                block.increase_the_time_2 = 0
                block.save_block()

    else:
        if not (int(time.time()) - block.raund_2_starting_time) < block.raund_2_time:
            if not block.increase_the_time_2 == 3:
                block.increase_the_time_2 += 1
                block.decrease_the_time_2 = 0
                block.save_block()



from setuptools import setup


setup(name='decentra_network',
version='0.8.4',
description="""This is an open source decentralized application network. In this network, you can develop and publish decentralized applications.""",
long_description=open("README.md", "r").read(),
long_description_content_type='text/markdown',
url='https://decentra-network.github.io/Decentra-Network/',
author='Decentra Network Developers',
author_email='atadogan06@gmail.com',
license='MPL-2.0',
packages=["", "accounts", "app", "blockchain", "blockchain.block", "blockchain.candidate_block", "consensus", "gui_lib", "gui_lib.libs", "gui_lib.libs.baseclass", "lib", "node", "transactions", "wallet"],
include_package_data=True,
package_dir={'':'src'},
entry_points = {
    'console_scripts': ['dngui = gui:start', 'dncli=cli:start', 'dnapi=api:start'],
},
python_requires=">=3.6",
zip_safe=False)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


class candidate_block:
    """
    The candidate_block class.

    candidate_block class consists of 2 elements:
      * A list element for candidate_blocks
      * A list element for candidate_block_hashes
    """

    def __init__(self, candidate_blocks=[], candidate_block_hashes=[]):

        self.candidate_blocks = candidate_blocks
        self.candidate_block_hashes = candidate_block_hashes



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

CONFIG_PATH = 'db/config.json'
SETTING_PATH = 'db/settings.json'

CONNECTED_NODE_PATH = 'db/connected_node.json'
UNL_NODES_PATH = 'db/unl_nodes.json'

TEMP_BLOCK_PATH = 'db/temp_block.decentra_network'
TEMP_ACCOUNTS_PATH = 'db/temp_accounts.decentra_network'
TEMP_ACCOUNTS_PART_PATH = 'db/temp_accounts_part.decentra_network'
TEMP_BLOCKSHASH_PATH = 'db/temp_blockshash.decentra_network'
TEMP_BLOCKSHASH_PART_PATH = 'db/temp_blockshash_part.decentra_network'

TEMP_CANDIDATE_BLOCKS_PATH = 'db/temp_candidate_blocks.decentra_network'

LOADING_BLOCK_PATH = 'db/loading_block.decentra_network'

BLOCKS_PATH = 'db/blocks/'


WALLETS_PATH = 'db/wallet_list.json'


MY_TRANSACTION_PATH = 'db/my_transaction.decentra_network'


# Export
MY_TRANSACTION_EXPORT_PATH = 'db/my_transaction.csv'




#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from transactions.save_to_my_transaction import GetMyTransaction


def PrintTransactions():
    print("\n")
    print(*[i.__dict__ for i in GetMyTransaction()], sep = "\n\n")
    print("\n")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from getpass import getpass

from wallet.wallet import Wallet_Create


def create_a_wallet(password=None):
    """
    Creates a wallet.
    """

    if password is None:
        password = getpass("Password: ")
    Wallet_Create(password)
    del password
